@@ -6,24 +6,27 @@
 # https://learn.microsoft.com/en-us/windows/win32/api/psapi/
 # ...
 
+from .enums import MemoryAllocationStatesEnum, MemoryProtectionsEnum, MemoryTypesEnum
 from .types import MEMORY_BASIC_INFORMATION, SYSTEM_INFO, WNDENUMPROC
 from .util import get_c_type_of
-from ctypes import POINTER, byref, c_uint32, c_void_p, create_unicode_buffer, sizeof, windll, wintypes
-from typing import Generator, Type, TypeVar, Union
+from typing import Generator, Optional, Tuple, Type, TypeVar, Union
+
+import ctypes
+import ctypes.wintypes
 
 # Load the libraries.
-kernel32 = windll.LoadLibrary("kernel32.dll")
-user32 = windll.LoadLibrary("user32.dll")
+kernel32 = ctypes.windll.LoadLibrary("kernel32.dll")
+user32 = ctypes.windll.LoadLibrary("user32.dll")
 
 # Set the argtypes to prevent ArgumentError.
 kernel32.VirtualQueryEx.argtypes = (
-    wintypes.HANDLE, wintypes.LPCVOID, POINTER(MEMORY_BASIC_INFORMATION), c_uint32
+    ctypes.wintypes.HANDLE, ctypes.wintypes.LPCVOID, ctypes.POINTER(MEMORY_BASIC_INFORMATION), ctypes.c_uint32
 )
 
 
 # Get the user's system information.
 system_information = SYSTEM_INFO()
-kernel32.GetSystemInfo(byref(system_information))
+kernel32.GetSystemInfo(ctypes.byref(system_information))
 
 
 T = TypeVar("T")
@@ -47,10 +50,11 @@ def GetMemoryRegions(process_handle: int) -> Generator[dict, None, None]:
 
     while current_address < mem_region_end:
         region = MEMORY_BASIC_INFORMATION()
-        kernel32.VirtualQueryEx(process_handle, current_address, byref(region), sizeof(region))
+        kernel32.VirtualQueryEx(process_handle, current_address, ctypes.byref(region), ctypes.sizeof(region))
+
+        yield {"address": current_address, "size": region.RegionSize, "struct": region}
 
         current_address += region.RegionSize
-        yield {"address": current_address, "size": region.RegionSize, "info_struct": region}
 
 
 def GetProcessHandle(access_right: int, inherit: bool, pid: int) -> int:
@@ -73,10 +77,10 @@ def GetProcessIdByWindowTitle(window_title: str) -> int:
     """
     Return the process ID by querying a window title.
     """
-    result = c_uint32(0)
+    result = ctypes.c_uint32(0)
 
     string_buffer_size = len(window_title) + 2  # (+2) for the next possible character of a title and the NULL char.
-    string_buffer = create_unicode_buffer(string_buffer_size)
+    string_buffer = ctypes.create_unicode_buffer(string_buffer_size)
 
     def callback(hwnd, size):
         """
@@ -92,7 +96,7 @@ def GetProcessIdByWindowTitle(window_title: str) -> int:
 
         # Compare the window titles and get the process ID.
         if window_title == string_buffer.value:
-            user32.GetWindowThreadProcessId(hwnd, byref(result))
+            user32.GetWindowThreadProcessId(hwnd, ctypes.byref(result))
             return False
 
         # Indicate it must continue enumeration.
@@ -117,12 +121,80 @@ def ReadProcessMemory(
     if pytype not in [bool, int, float, str, bytes]:
         raise ValueError("The type must be bool, int, float, str or bytes.")
 
-    data = get_c_type_of(pytype, int(bufflength))
-    kernel32.ReadProcessMemory(process_handle, c_void_p(address), byref(data), bufflength, None)
+    data = get_c_type_of(pytype, bufflength)
+    kernel32.ReadProcessMemory(process_handle, ctypes.c_void_p(address), ctypes.byref(data), bufflength, None)
 
     return str(data.value) if pytype is str else data.value
 
 
+def SearchAllMemory(
+    process_handle: int,
+    pytype: Type[T],
+    bufflength: int,
+    value: Union[bool, int, float, str, bytes],
+    progress_information: Optional[bool] = False,
+) -> Generator[Union[int, Tuple[int, dict]], None, None]:
+    """
+    Search the whole memory space, accessible to the process,
+    for the provided value, returning the found addresses.
+    """
+    if pytype not in [bool, int, float, str, bytes]:
+        raise ValueError("The type must be bool, int, float, str or bytes.")
+
+    # Get the target value as bytes.
+    target_value = get_c_type_of(pytype, bufflength)
+    target_value.value = value
+
+    target_value = bytes(target_value)
+
+    regions = list()
+    memory_total = 0
+
+    # Get the memory regions, computing the space size.
+    for region in GetMemoryRegions(process_handle):
+
+        # Only committed, non-shared and readable memory pages.
+        if region["struct"].State != MemoryAllocationStatesEnum.MEM_COMMIT.value: continue
+        if region["struct"].Type != MemoryTypesEnum.MEM_PRIVATE.value: continue
+        if region["struct"].Protect & MemoryProtectionsEnum.PAGE_READABLE.value == 0: continue
+
+        memory_total += region["size"]
+        regions.append(region)
+
+    checked_memory_size = 0
+
+    # Check each memory region used by the process.
+    for region in regions:
+        address, size = region["address"], region["size"]
+        region_data = (ctypes.c_byte * size)()
+
+        # Get data from the region.
+        kernel32.ReadProcessMemory(process_handle, ctypes.c_void_p(address), ctypes.byref(region_data), size, None)
+        region_data = bytes(region_data)
+
+        # Walk by the returned bytes, searching for the target value.
+        current_index = 0
+        result_index = 0
+
+        while current_index < size and result_index != -1:
+            result_index = region_data.find(target_value, current_index)
+
+            # Result equals (-1) means that there is no more match in the region data.
+            if result_index == -1: break
+            current_index = result_index + 1
+
+            found_address = address + result_index
+
+            extra_information = {
+                "memory_total": memory_total,
+                "progress": (checked_memory_size + current_index) / memory_total
+            }
+            yield (found_address, extra_information) if progress_information else found_address
+
+        # Compute the region size to the checked memory size.
+        checked_memory_size += size
+
+
 def WriteProcessMemory(
     process_handle: int,
     address: int,
@@ -136,9 +208,9 @@ def WriteProcessMemory(
     if pytype not in [bool, int, float, str, bytes]:
         raise ValueError("The type must be bool, int, float, str or bytes.")
 
-    data = get_c_type_of(pytype, int(bufflength))
+    data = get_c_type_of(pytype, bufflength)
     data.value = value.encode() if isinstance(value, str) else value
 
-    kernel32.WriteProcessMemory(process_handle, c_void_p(address), byref(data), bufflength, None)
+    kernel32.WriteProcessMemory(process_handle, ctypes.c_void_p(address), ctypes.byref(data), bufflength, None)
 
     return value
